// simple.frag

#version 120
#extension GL_EXT_geometry_shader3 : enable
#extension GL_EXT_gpu_shader3 : enable

uniform sampler2D textureImage;
uniform int textureMode;
uniform int fogMode;
uniform int lightNumber;
uniform int lightRecieve[7];
uniform int useTexAlpha;
uniform int dissolveMode;

uniform vec2 ensembleParamA;
uniform float ensembleParamB;
uniform vec2 ensembleParamC;

uniform float projInv11;
uniform float projInv15;

uniform int shadowVolumeMode;
uniform float edgeThreshold;

varying vec3 position;
varying vec3 normal;

//##FUNC_SMOOTHCLIPPING##

//##FUNC_ENSEMBLE_DITHERING##

//##FUNC_FOG_EFFECT##

//##FUNC_SPOT_EFFECT##

void main (void)
{
    vec4 texColor = vec4(1.0,1.0,1.0,1.0); 
    if (textureMode == 1)
	texColor = texture2D(textureImage, gl_TexCoord[0].st);
    
    float alpha = gl_FrontLightProduct[0].ambient[3];
    if (useTexAlpha == 1){
	alpha *= texColor[3];
	if (alpha == 0) discard;
    }
    
    //##APPLY_SMOOTHCLIPPING##

    vec3 fnormal = normalize(normal);

    vec3  eye = - normalize(position);
    float eyedot = dot(eye, fnormal);
    if (eyedot * eyedot < edgeThreshold){
	gl_FragColor = vec4(0, 0, 0, alpha);
	if (fogMode != 0) FogEffect();
	//##APPLY_ENSEMBLE_DITHERING##
	return;
    }

    //##APPLY_ENSEMBLE_DITHERING##
    
    if (alpha > 1.0) alpha = 1.0;

    // ׂ͂ĔFF
    float diffuseTotal = 0.0;
    float specularTotal = 0.0;

    vec3 viewVec = -normalize(position);
    
    for (int i = 0; i < lightNumber; i++){
	if (lightRecieve[i] == 0) continue;
	vec3 lightVec = gl_LightSource[i].position.xyz - position * gl_LightSource[i].position[3];
	float dis = length(lightVec);
	lightVec = normalize(lightVec);

	// fBt[Y 
	float diffuse = dot(lightVec, fnormal);
	
	if (diffuse > 0.0){
	    float spotAngleCos = dot(normalize(gl_LightSource[i].spotDirection),
				     -lightVec);
	    float lightFactor = GetSpotEffect(i, spotAngleCos);
	    
	    // W
	    lightFactor /= (gl_LightSource[i].constantAttenuation +
			    gl_LightSource[i].linearAttenuation * dis +
			    gl_LightSource[i].quadraticAttenuation * dis * dis);

	    //˃xNg 
	    vec3 reflectVec = reflect(-lightVec, fnormal);
	    float specular = 
		pow(max(dot(viewVec, reflectVec), 0.0),
		    gl_FrontMaterial.shininess) * lightFactor;
	    diffuseTotal += diffuse * gl_LightSource[i].diffuse.r * lightFactor;
	    specularTotal += specular * gl_LightSource[i].diffuse.r;
	}
    }
    
    float s = diffuseTotal * 0.8 + specularTotal;
    //if (s > 1.0) s = 1.0;
    
    vec4 ambient = gl_FrontMaterial.ambient * float(shadowVolumeMode);
    
    if (s < 0.2){
	gl_FragColor = ambient;
    }
    else if (s < 0.75){
	gl_FragColor = ambient +
	    gl_FrontMaterial.diffuse;
    }
    else{
	gl_FragColor = ambient + 
	    gl_FrontMaterial.diffuse + 
	    gl_FrontMaterial.specular;
    }
    
    gl_FragColor *= texColor;
    
    gl_FragColor[3] = alpha;

    if (fogMode != 0) FogEffect();
}
