// Gas.vert

#version 130
#extension GL_EXT_gpu_shader4 : enable
#extension GL_ARB_explicit_attrib_location : enable

uniform int lightNumber;
uniform int lightRecieve[7];
uniform float scatter;
uniform float factorAmb;

in float pointSize;
in int subIndex;

in vec3 tangent;
in vec3 binormal;

out vec3 positionIn;
out float pointSizeIn;
out int   subIndexIn;
out vec3 normalIn;
out vec3 tangentIn;
out vec3 binormalIn;
out vec4 diffuseAndAmbientIn;

//##FUNC_SPOT_EFFECT##

void main(void)
{
    vec3 position = vec3(gl_ModelViewMatrix * gl_Vertex);

    gl_Position = /* gl_ModelViewProjectionMatrix * */ gl_Vertex;

    vec3 normal = gl_Normal;
    normalIn = normal;
    tangentIn = tangent;
    binormalIn = cross(normal, tangent);

    gl_FrontColor = gl_Color;
    pointSizeIn = pointSize;
    positionIn  = position;
    normalIn    = /*gl_NormalMatrix * */gl_Normal;
    subIndexIn  = subIndex;

    float diffuseTotal = 0.0;
    for (int i = 0; i < lightNumber; i++){
	if (lightRecieve[i] == 0) continue;
	vec3 lightVec = gl_LightSource[i].position.xyz - position * gl_LightSource[i].position[3];
	float dis = length(lightVec);
	lightVec = normalize(lightVec);

	float strength = gl_LightSource[i].diffuse.r;
	float spotAngleCos = dot(normalize(gl_LightSource[i].spotDirection),
				 -lightVec);
	float spotEffect = GetSpotEffect(i, spotAngleCos);

	// W 
	float attenuation = 1.0 / (gl_LightSource[i].constantAttenuation +
				   gl_LightSource[i].linearAttenuation * dis +
				   gl_LightSource[i].quadraticAttenuation * dis * dis);
	
	vec3 viewVec = normalize(-position);
	// fBt[Y 
	float diffuse = dot(lightVec, viewVec);

	diffuse = diffuse * scatter + (1.0f - factorAmb);
	
	diffuse *= spotEffect * attenuation * strength;
	
	diffuseTotal += diffuse;
    }
    
    float ambient = gl_LightSource[0].ambient.r;
    
    diffuseAndAmbientIn = vec4(diffuseTotal + ambient*factorAmb,
			       diffuseTotal + ambient*factorAmb,
			       diffuseTotal + ambient*factorAmb, 1);
}
