// UseNormalPS.frag

#version 120
#extension GL_EXT_gpu_shader4 : enable

uniform sampler2D textureImage0;
uniform sampler2D textureImage1;
uniform int fogMode;
uniform int lightNumber;
uniform int lightRecieve[7];
uniform float clipThreshold;
uniform int dissolveMode;

uniform vec2 ensembleParamA;
uniform float ensembleParamB;
uniform vec2 ensembleParamC;

uniform float projInv11;
uniform float projInv15;

varying vec3 position;
varying vec3 viewVec;
varying vec3 normal;
varying vec3 binormal;
varying vec3 tangent;

//##FUNC_SMOOTHCLIPPING##

//##FUNC_ENSEMBLE_DITHERING##

//##FUNC_FOG_EFFECT##

//##FUNC_SPOT_EFFECT##

void main (void)
{
    vec4 texColorBase = texture2D(textureImage1, gl_TexCoord[0].st);
    vec4 texNormal    = texture2D(textureImage0, gl_TexCoord[0].st);

    float alpha = gl_FrontLightProduct[0].ambient[3] * gl_Color[3];
    
    alpha = texColorBase[3] * alpha;
    if (alpha < clipThreshold) discard;

    //##APPLY_SMOOTHCLIPPING##

    //##APPLY_ENSEMBLE_DITHERING##
    
    //if (alpha > 1.0) alpha = 1.0;
    
    // ׂ͂ĔFF
    float diffuseTotal = 0.0;
    float specularTotal = 0.0;

    float r = texNormal.r;
    float g = texNormal.g;
    float b = texNormal.b;

    vec3 fnormal = normalize(
	normal * (b * 2.0f - 1.0f) +
	tangent * (r * 2.0f - 1.0f) +
	binormal * (g * 2.0f - 1.0f) );

    //vec3 viewVec = -normalize(position);

    for (int i = 0; i < lightNumber; i++){
	if (lightRecieve[i] == 0) continue;
	vec3 lightVec = gl_LightSource[i].position.xyz - position * gl_LightSource[i].position[3];
	float dis = length(lightVec);
	lightVec = normalize(lightVec);

	// fBt[Y 
	float diffuse = dot(lightVec, fnormal);
	
	if (diffuse > 0.0){
	    float spotAngleCos = dot(normalize(gl_LightSource[i].spotDirection),
				     -lightVec);
	    float lightFactor = GetSpotEffect(i, spotAngleCos);
	    
	    // W
	    lightFactor /= (gl_LightSource[i].constantAttenuation +
			    gl_LightSource[i].linearAttenuation * dis +
			    gl_LightSource[i].quadraticAttenuation * dis * dis);

	    //˃xNg 
	    vec3 reflectVec = reflect(-lightVec, fnormal);
	    float specular = 
		pow(max(dot(viewVec, reflectVec), 0.0),
		    10.0/*gl_FrontMaterial.shininess*/) * lightFactor;
	    if (specular < 0) specular = 0;
	    if (specular > 1.0f) specular = 1.0f;
	    diffuseTotal += diffuse * gl_LightSource[i].diffuse.r * lightFactor;
	    specularTotal += specular * gl_LightSource[i].diffuse.r;
	}
    }

    gl_FragColor = (gl_FrontLightProduct[0].ambient + 
		    gl_FrontMaterial.diffuse * diffuseTotal +
		    gl_FrontMaterial.specular * specularTotal);

    gl_FragColor *= texColorBase * gl_Color;

    //##APPLY_ENSEMBLE_DITHERING##
    
    //##APPLY_FOG_EFFECT##
    
    gl_FragColor[3] = alpha;
}
