// UseNormalPS.geom

// SetParameter GL_GEOMETRY_INPUT_TYPE_EXT=GL_POINTS
// SetParameter GL_GEOMETRY_OUTPUT_TYPE_EXT=GL_TRIANGLE_STRIP
// SetParameter GL_GEOMETRY_VERTICES_OUT_EXT=4

#version 120
#extension GL_EXT_geometry_shader4 : enable
#extension GL_EXT_gpu_shader4 : enable

uniform float aspectAngleFactor; 

varying in vec3 positionIn[];
varying in int  subIndexIn[];
varying in float pointSizeIn[];

varying in vec3 viewVecIn[];
varying in vec3 normalIn[];
varying in vec3 binormalIn[];
varying in vec3 tangentIn[];

varying out vec3 position;
varying out vec3 viewVec;
varying out vec3 normal;
varying out vec3 binormal;
varying out vec3 tangent;

void main(void){

    vec4 offset[4] = vec4[]
	(vec4(-1, -1, 0,0), vec4(1, -1, 0,0), vec4(-1, 1, 0,0), vec4(1, 1, 0,0));

    int subIndex = subIndexIn[0];
    int atrasGrid = subIndex / 256;
    if (atrasGrid == 0) atrasGrid = 1;
    subIndex = subIndex % 256;

    float atrasWidth = 1.0 / atrasGrid;
    float atrasHeight = 1.0 / atrasGrid;

    vec4 texCoord[4];
    int offsetX = subIndex % atrasGrid;
    int offsetY = subIndex / atrasGrid;
    texCoord[0] = vec4(offsetX * atrasWidth , offsetY * atrasHeight, 0, 0);
    texCoord[1] = vec4((offsetX + 1) * atrasWidth, offsetY * atrasHeight, 0, 0);
    texCoord[2] = vec4(offsetX * atrasWidth, (offsetY + 1) * atrasHeight, 0, 0);
    texCoord[3] = vec4((offsetX + 1) * atrasWidth, (offsetY + 1) * atrasHeight, 0, 0);
   
    float psize = pointSizeIn[0] * aspectAngleFactor;

    int length = gl_PositionIn.length();

    for(int i = 0; i < length; i++){
	viewVec = viewVecIn[i];
	normal = normalIn[i];
	binormal = binormalIn[i];
	tangent = tangentIn[i];

	gl_FrontColor = gl_FrontColorIn[i];
	gl_TexCoord[0] = texCoord[0];
	gl_Position = gl_PositionIn[i] + offset[0] * psize;
	position = positionIn[i] + vec3(offset[0]*psize);
	EmitVertex();
	gl_TexCoord[0] = texCoord[1];
	gl_Position = gl_PositionIn[i] + offset[1] * psize;
	position = positionIn[i] + vec3(offset[1]*psize);
	EmitVertex();
	gl_TexCoord[0] = texCoord[2];
	gl_Position = gl_PositionIn[i] + offset[2] * psize;
	position = positionIn[i] + vec3(offset[2]*psize);
	EmitVertex();
	gl_TexCoord[0] = texCoord[3];
	gl_Position = gl_PositionIn[i] + offset[3] * psize;
	position = positionIn[i] + vec3(offset[3]*psize);
	EmitVertex();
    }
    
    EndPrimitive();
}
