// simple.frag

#extension GL_EXT_gpu_shader4 : enable

uniform sampler2D textureImage;
uniform vec4 levelMin;
uniform vec4 levelMax;
uniform vec4 exposure;
uniform vec4 gamma;
uniform int hueOffset;
uniform int saturation;
uniform int blendMode;
uniform float alpha;

void main (void)
{
    vec4 texColor = texture2D(textureImage, gl_TexCoord[0].st);

    texColor[0] += 0.0001;
    texColor[1] += 0.0001;
    texColor[2] += 0.0001;
    
    float texAlpha = texColor[3];
    float alphaValue = gl_Color[3];
    float finAlpha = alpha * alphaValue;
    //float finAlpha = 0.5;
    
    texColor = (texColor-levelMin) / (levelMax - levelMin);
    
    if (hueOffset != 0 || saturation != 0){
	float R = texColor.r;
	float G = texColor.g;
	float B = texColor.b;

	R = max (R, 0.0);
	G = max (G, 0.0);
	B = max (B, 0.0);
	
	R = min (R, 1.0);
	G = min (G, 1.0);
	B = min (B, 1.0);
	
	if (! (R == 1 && G == 1 && B == 1)){

	    float max = R;
	    int maxType = 0;
	    if (G > max){ max = G; maxType = 1; }
	    if (B > max){ max = B; maxType = 2; }
	
	    float min = R;
	    if (G < min){ min = G; }
	    if (B < min){ min = B; }
	
	    float H = 0.0, S = 0.0, V = 0.0;
	    if      (max == 0.0)  { H = 0.0; S = 0.0; V = 0.0; }
	    else if (max == min){ H = 0.0; S = 0.0; V = max;}
	    else {
		V = max;
		S = (float(max - min)) / max;

		if (maxType == 0)
		    H = (float(G - B))/(max - min) + 0.0; 
		else if (maxType == 1)
		    H = (float(B - R))/(max - min) + 2.0; 
		else if (maxType == 2)
		    H = (float(R - G))/(max - min) + 4.0; 
	    }
	
	    S += float(saturation)/100.0;
	    if (S > 1.0) S = 1.0;
	    if (S < 0.0) S = 0.0;
	
	    H += float(hueOffset)/60.0;
	
	    while (H < 0.0) H += 6.0;
	    H = mod(H, 6.0); // mod : GLSL̑gݍ݊֐
	    //int Hi = (int(H)) % 6;
	    int Hi = int(H);
	    float f = float(H) - float(Hi);

	    if (Hi == 0){
		R = V;
		G = V * ( 1.0 - ( 1.0 - f) * S);
		B = V * ( 1.0 - S);
	    }
	    else if (Hi == 1){
		R = V * ( 1.0 - f * S);
		G = V;
		B = V * ( 1.0 - S);
	    }
	    else if (Hi == 2){
		R = V * ( 1.0 - S);
		G = V;
		B = V * ( 1.0 - ( 1.0 - f) * S);
	    }
	    else if (Hi == 3){
		R = V * ( 1.0 - S);
		G = V * ( 1.0 - f * S);
		B = V;
	    }
	    else if (Hi == 4){
		R = V * ( 1.0 - ( 1.0 - f) * S);
		G = V * ( 1.0 - S);
		B = V;
	    }
	    else if (Hi == 5){
		R = V;
		G = V * ( 1.0 - S);
		B = V * ( 1.0 - f * S);
	    }
	}

	texColor = vec4(R, G, B, texAlpha);
    }
    else{
    	float R = texColor.r;
	float G = texColor.g;
	float B = texColor.b;
	texColor = vec4(R, G, B, texAlpha);	
    }
    
    if (gamma[0] != 0.0) 
	texColor = pow(texColor, gamma);
    
    if (blendMode == 0)
	gl_FragColor = (1.0-finAlpha) * gl_FragColor + finAlpha * texColor;
    else
	gl_FragColor = gl_FragColor + finAlpha * texColor;
    return;
}
